@@ -198,6 +198,19 @@ void File_Aac::Streams_Update()
 //---------------------------------------------------------------------------
 void File_Aac::Streams_Finish()
 {
+    std::bitset<32> sect_cb_used;
+    for (int g = 0; g < num_window_groups; g++)
+    {
+        for (int8u i = 0; i < num_sec[g]; i++)
+        {
+            if (sect_cb[g][i] == 13) //NOISE_HCB
+            {
+                Fill(Stream_Audio, 0, Audio_Format_Settings, "PNS");
+                break;
+            }
+        }
+    }
+
     switch(Mode)
     {
         case Mode_ADIF    :
