@@ -76,6 +76,51 @@ static const char*  Opus_ChannelLayout[Opus_ChannelLayout_Max]=
     "L R C SL SR BL BR LFE",
 };
 
+//---------------------------------------------------------------------------
+static const string Opus_Config(int8u config)
+{
+    struct ConfigDescription {
+        const int8u start;
+        const int8u end;
+        const char* mode;
+        const char* bandwidth;
+        const std::vector<int8u> frame_sizes_ms;
+    };
+
+    const ConfigDescription config_table[] = {
+        {0, 3, "SILK-only", "NB", {10, 20, 40, 60}},
+        {4, 7, "SILK-only", "MB", {10, 20, 40, 60}},
+        {8, 11, "SILK-only", "WB", {10, 20, 40, 60}},
+        {12, 13, "Hybrid", "SWB", {10, 20}},
+        {14, 15, "Hybrid", "FB", {10, 20}},
+        {16, 19, "CELT-only", "NB", {2, 5, 10, 20}},
+        {20, 23, "CELT-only", "WB", {2, 5, 10, 20}},
+        {24, 27, "CELT-only", "SWB", {2, 5, 10, 20}},
+        {28, 31, "CELT-only", "FB", {2, 5, 10, 20}}
+    };
+
+    if (config > 31)
+        return "";
+
+    for (const auto& config_it : config_table) {
+        if (config >= config_it.start && config <= config_it.end) {
+            int8u index = config - config_it.start;
+            if (index < config_it.frame_sizes_ms.size())
+                return string(config_it.mode) + ", " + config_it.bandwidth + ", " + to_string(config_it.frame_sizes_ms[index]) + " ms";
+        }
+    }
+    return "";
+}
+
+//---------------------------------------------------------------------------
+static const char* Opus_c[4] =
+{
+    "1 frame in the packet",
+    "2 frames in the packet, each with equal compressed size",
+    "2 frames in the packet, with different compressed sizes",
+    "an arbitrary number of frames in the packet"
+};
+
 //***************************************************************************
 // Constructor/Destructor
 //***************************************************************************
@@ -124,13 +169,19 @@ void File_Opus::Identification()
     int8u Opus_version_id, ch_count, ch_map;
     int16u preskip;
     int32u sample_rate;
-    Get_UTF8(8,opus_version,                                    "opus_codec_id");
+    if (!(FromMP4 || FromIamf))
+        Get_UTF8(8, opus_version,                               "opus_codec_id");
     Get_L1 (Opus_version_id,                                    "opus_version_id");
     Get_L1 (ch_count,                                           "channel_count");
-    Get_L2 (preskip,                                            "preskip");
-    Get_L4 (sample_rate,                                        "rate");
-    Skip_L2(                                                    "ouput_gain");
+    Get_X2 (preskip,                                            "preskip");
+    Get_X4 (sample_rate,                                        "rate");
+    Skip_X2(                                                    "ouput_gain");
     Get_L1 (ch_map,                                             "channel_map");
+    if (FromIamf && ch_map) {
+        Identification_Done = true;
+        Trusted_IsNot("Opus ID Header does not conform with ChannelMappingFamily = 0 when in IAMF");
+        return;
+    }
     if (ch_map)
     {
         Skip_L1(                                                "Stream count (N)");
@@ -149,14 +200,23 @@ void File_Opus::Identification()
         Fill(Stream_Audio, 0, Audio_Format, "Opus");
         Fill(Stream_Audio, 0, Audio_Codec, "Opus");
 
-        if (!opus_version.empty())
+        if (!opus_version.empty() || FromMP4 || FromIamf)
         {
             Fill(Stream_Audio, 0, Audio_SamplingRate, sample_rate?sample_rate:48000);
-            Fill(Stream_Audio, 0, Audio_Channel_s_, ch_count);
+            if (!FromIamf)
+                Fill(Stream_Audio, 0, Audio_Channel_s_, ch_count);
         }
 
-        switch (ch_map)
-        {
+        if (!FromIamf) {
+            //      0       Mono, L/R stereo                    [RFC7845, Section 5.1.1.1][RFC8486, Section 5]
+            //      1       1 - 8 channel surround              [RFC7845, Section 5.1.1.2][RFC8486, Section 5]
+            //      2       Ambisonics as individual channels   [RFC8486, Section 3.1]
+            //      3       Ambisonics with demixing matrix     [RFC8486, Section 3.2]
+            //    4 - 239   Unassigned
+            //  240 - 254   Experimental use                    [RFC8486, Section 6]
+            //     255      Discrete channels                   [RFC7845, Section 5.1.1.3][RFC8486, Section 5]
+            switch (ch_map)
+            {
             case 0 : // Mono/Stereo
                     if (ch_count>2)
                         break; // Not in spec
@@ -176,6 +236,7 @@ void File_Opus::Identification()
                     }
                     break;
             default: ; //Unknown
+            }
         }
 
     FILLING_END();
@@ -189,9 +250,73 @@ void File_Opus::Stream()
 {
     Element_Name("Stream");
 
-    Skip_XX(Element_Size,                                       "Data");
+    Element_Begin1("TOC");
+    int8u config, c;
+    bool s;
+    BS_Begin();
+    Get_S1(5, config,                                           "config"); Param_Info1(Opus_Config(config));
+    Get_SB(s,                                                   "s (Stereo)");
+    Get_S1(2, c,                                                "c (Code)"); Param_Info1(Opus_c[c]);
+    BS_End();
+    Element_End0();
 
-    Finish("Opus");
+    switch (c) {
+    case 0:
+        Skip_XX(Element_Size - 1,                               "Compressed frame 1");
+        break;
+    case 1:
+        Skip_XX((Element_Size - 1) / 2,                         "Compressed frame 1");
+        Skip_XX((Element_Size - 1) / 2,                         "Compressed frame 2");
+        break;
+    case 2:
+    {
+        int8u N1_1{}, N1_2{};
+        int16u size{};
+        Get_L1(N1_1,                                            "N1");
+        if (N1_1 < 252)
+            size = N1_1;
+        else {
+            Get_L1(N1_2,                                        "N1");
+            size = N1_2 * 4 + N1_1;
+        }
+        Skip_XX(size,                                           "Compressed frame 1");
+        Skip_XX(Element_Size - Element_Offset,                  "Compressed frame 2");
+        break;
+    }
+    default:
+        Skip_XX(Element_Size,                                   "Data");
+        break;
+    }
+
+    if (!FromIamf)
+        Finish("Opus");
+}
+
+//---------------------------------------------------------------------------
+void File_Opus::Get_X2(int16u& Info, const char* Name)
+{
+    if (FromMP4 || FromIamf)
+        Get_B2(Info, Name);
+    else
+        Get_L2(Info, Name);
+}
+
+//---------------------------------------------------------------------------
+void File_Opus::Get_X4(int32u& Info, const char* Name)
+{
+    if (FromMP4 || FromIamf)
+        Get_B4(Info, Name);
+    else
+        Get_L4(Info, Name);
+}
+
+//---------------------------------------------------------------------------
+void File_Opus::Skip_X2(const char* Name)
+{
+    if (FromMP4 || FromIamf)
+        Skip_B2(Name);
+    else
+        Skip_L2(Name);
 }
 
 //***************************************************************************
