@@ -32,6 +32,8 @@
 #include "MediaInfo/Text/File_Pac_Codepages.h"
 #include "MediaInfo/MediaInfo_Config_MediaInfo.h"
 #include "MediaInfo/TimeCode.h"
+#include <algorithm>
+#include <cmath>
 #include <limits>
 #include <map>
 #include <set>
@@ -88,23 +90,37 @@ void File_Pac::Streams_Accept()
     Fill(Stream_General, 0, General_Format, "PAC");
     Stream_Prepare(Stream_Text);
     Fill(Stream_Text, 0, Text_Format, "PAC");
-    Fill(Stream_Text, 0, Text_FrameRate, 24);
 }
 
 //---------------------------------------------------------------------------
 void File_Pac::Streams_Finish()
 {
     #if MEDIAINFO_ADVANCED
-        float64 FrameRate_F = Video_FrameRate_Rounded(Config->File_DefaultFrameRate_Get());
-        if (!FrameRate_F)
-            FrameRate_F = 24; // Files I have are all with frame rate of 24 fps
-        int64s FrameRate_Int = float64_int64s(FrameRate_F);
-        const uint32_t FrameMax = (uint32_t)(FrameRate_Int - 1);
+        const float64 FrameRate_FromConfig = Video_FrameRate_Rounded(Config->File_DefaultFrameRate_Get());
+        const auto FrameMax = (FrameRate_FromConfig && FrameRate_FromConfig >= FF_Max + 1)
+            ? ((const uint32_t)ceil(FrameRate_FromConfig) - 1)
+            : (FF_Max <= 23 ? 23 : FF_Max <= 24 ? 24 : FF_Max <= 29 ? 29 : FF_Max); // Files I have are all with frame rate of 24 fps
+        const auto FrameRate_FromFrameMax = (const float64)FrameMax + 1;
+        const auto FrameRate = FrameRate_FromConfig ? FrameRate_FromConfig : FrameRate_FromFrameMax;
+        const auto FrameRateIsGuessed = FrameRate_FromConfig == 0;
+        const auto DropFrame = Config->File_DefaultTimeCodeDropFrame_Get() == 1;
+        const auto FrameRate_Diff = float64_int64s(FrameRate_FromFrameMax / (FrameRate_FromFrameMax - FrameRate));
+        const auto Is1001 = FrameRate_Diff == 1001;
     #else //MEDIAINFO_ADVANCED
-        const uint32_t FrameMax = 23;
+        const uint32_t FrameMax = FF_Max <= 23 ? 23 : FF_Max <= 24 ? 24 : FF_Max <= 30 ? 30 : FF_Max;
+        const auto FrameRate_FromFrameMax = (const float32)FrameMax + 1;
+        const auto FrameRate = FrameRate_FromFrameMax;
+        const auto FrameRateIsGuessed = true;
+        const auto DropFrame = false;
+        const auto Is1001 = false;
     #endif //MEDIAINFO_ADVANCED
+    Fill(Stream_Text, 0, Text_FrameRate, Is1001 ? (FrameRate_FromFrameMax / 1.001) : FrameRate, FrameRateIsGuessed?0:3); // Default without decimal as a hint that this is a guessed value
     Time_Start_Command.SetFramesMax(FrameMax);
+    Time_Start_Command.SetDropFrame(DropFrame);
+    Time_Start_Command.Set1001fps(Is1001);
     Time_End_Command.SetFramesMax(FrameMax);
+    Time_End_Command.SetDropFrame(DropFrame);
+    Time_End_Command.Set1001fps(Is1001);
     TimeCode Delay_TC;
     TimeCode Offset;
     if (Time_Delay.IsValid()) {
@@ -117,30 +133,32 @@ void File_Pac::Streams_Finish()
         Offset.SetSeconds(0);
         Offset.SetFrames(0);
     }
-    int64s Duration = (Time_End_Command - Time_Start_Command).ToMilliseconds();
+    int64s Duration = (Time_End_Command - Offset).ToMilliseconds();
     Fill(Stream_General, 0, General_Duration, Duration);
     Fill(Stream_Text, 0, Text_Duration, Duration);
-    Fill(Stream_Text, 0, Text_Duration_Start_Command, (int64s)(Time_Start_Command - Offset).ToMilliseconds());
+    Fill(Stream_Text, 0, Text_Duration_Start_Command, (int64s)(Time_Start_Command).ToMilliseconds());
     Fill(Stream_Text, 0, Text_TimeCode_FirstFrame, Time_Start_Command.ToString());
-    Fill(Stream_Text, 0, Text_Duration_End_Command, (int64s)(Time_End_Command - Offset).ToMilliseconds());
+    Fill(Stream_Text, 0, Text_Duration_End_Command, (int64s)(Time_End_Command).ToMilliseconds());
     TimeCode LastFrame = Time_End_Command;
     --LastFrame;
     Fill(Stream_Text, 0, Text_TimeCode_LastFrame, LastFrame.ToString());
     if (Time_Start.IsValid()) {
         Time_Start.SetFramesMax(FrameMax);
-        Fill(Stream_Text, 0, Text_Duration_Start, (int64s)(Time_Start - Offset).ToMilliseconds());
+        Time_Start.SetDropFrame(DropFrame);
+        Time_Start.Set1001fps(Is1001);
+        Fill(Stream_Text, 0, Text_Duration_Start, (int64s)(Time_Start).ToMilliseconds());
     }
     if (Time_End.IsValid()) {
         Time_End.SetFramesMax(FrameMax);
-        Fill(Stream_Text, 0, Text_Duration_End, (int64s)(Time_End - Offset).ToMilliseconds());
+        Time_End.SetDropFrame(DropFrame);
+        Time_End.Set1001fps(Is1001);
+        Fill(Stream_Text, 0, Text_Duration_End, (int64s)(Time_End).ToMilliseconds());
     }
     if (Time_End.IsValid() && Time_Start.IsValid()) {
         auto Start2End = (int64s)(Time_End - Time_Start).ToMilliseconds();
         Fill(Stream_Text, 0, Text_Duration_Start2End, Start2End);
     }
-    if (Delay_TC.IsValid()) {
-        Fill(Stream_Text, 0, Text_Delay, (int64s)Delay_TC.ToMilliseconds());
-    }
+    Fill(Stream_Text, 0, Text_Delay, (int64s)(Delay_TC.IsValid()?Delay_TC:Offset).ToMilliseconds());
 
     if (Frame_Count) {
         Fill(Stream_Text, 0, Text_Events_Total, Frame_Count - EmptyCount);
@@ -203,6 +221,9 @@ void File_Pac::Header_Parse()
         Get_L2 (HHMM,                                           "HHMM");
         Get_L2 (SSFF,                                           "SSFF");
         TimeCode TC(HHMM / 100, HHMM % 100, SSFF / 100, SSFF % 100, 99);
+        if (FF_Max < TC.GetFrames()) {
+            FF_Max = TC.GetFrames();
+        }
         if (!TC.IsValid()) {
             Reject();
         }
